<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

namespace Redis\Model;

use P4\Connection\ConnectionInterface;
use P4\Spec\Exception\NotFoundException as SpecNotFoundException;
use Redis\RedisService;
use Users\Model\IUser;
use Users\Model\User;

/**
 * DAO to handle finding/saving users.
 * @package Redis\Model
 */
class UserDAO extends PluralAbstractDAO
{
    // The main key prefix to reference an individual record, for example 'user^fred'
    const CACHE_KEY_PREFIX = IUser::USER . RedisService::SEPARATOR;
    // The Perforce class that handles users
    const MODEL = User::class;
    // The Key used by userDAO to know if it has been pre populated.
    const POPULATED_STATUS = IUser::USER . "-" . AbstractDAO::POPULATED_STATUS;
    // The key for the verify status of the user dataset
    const VERIFY_STATUS = IUser::USER . "-" . AbstractDAO::VERIFY_STATUS;
    // When populating maximum set to null to fetch all records. The user model does not support FETCH_AFTER
    const FETCH_MAXIMUM = null;

    /**
     * Call the parent fetchById first converting the id to the correct case
     * @param string                    $id         the id
     * @param ConnectionInterface|null  $connection the connection
     * @return mixed
     * @throws SpecNotFoundException
     */
    public function fetchById($id, ConnectionInterface $connection = null)
    {
        // Check for not null explicitly (0 is a possible numeric id)
        if ($id !== null) {
            return parent::fetchById($id, $connection);
        } else {
            throw new SpecNotFoundException("Cannot fetch user $id. Record does not exist.");
        }
    }

    /**
     * Converts the id to a normalized value.
     * @param string                $id         the id
     * @param ConnectionInterface   $connection connection details
     * @return string the normalized id. If the connection specifies non case sensitive that a lowercase version of
     * the id is returned, otherwise the id is returned unchanged.
     */
    protected function normalizeId($id, ConnectionInterface $connection = null)
    {
        return $this->getCaseSpecificId($id, $connection);
    }
}
