<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */
namespace TestIntegration\Listener;

use Application\Config\IDao;
use Application\Connection\ConnectionFactory;
use Application\Log\SwarmLogger;
use Events\Listener\AbstractEventListener;
use Reviews\Model\Review;
use TestIntegration\Model\TestRun as Model;
use Zend\EventManager\Event;
use Record\Exception\NotFoundException;

/**
 * Class TestRun, listens to TestRun events raised based on queue tasks
 * @package TestIntegration\Listener
 */
class TestRun extends AbstractEventListener
{

    /**
     * Listener for TestRun add/update. Updates the overall test status on a review if the
     * TestRun changed is for the latest version
     * @param Event $event  Zend event
     * @return bool true if a review was updated, false otherwise
     */
    public function update(Event $event)
    {
        $updated = true;
        $data    = $event->getParam('data');
        $id      = $data[Model::FIELD_CHANGE];
        $version = $data[Model::FIELD_VERSION];
        try {
            $p4Admin = $this->services->get(ConnectionFactory::P4_ADMIN);
            $review  = Review::fetch($id, $p4Admin);
            if ($version == $review->getHeadVersion()) {
                $dao    = $this->services->get(IDao::TEST_RUN_DAO);
                $status = $dao->calculateTestStatus($review->getTestRuns(), $p4Admin);
                if ($status) {
                    $review->setTestStatus($status)->save();
                }
            }
        } catch (NotFoundException $e) {
            $updated = false;
            $logger  = $this->services->get(SwarmLogger::SERVICE);
            $logger->err(sprintf("[%s]: Review [%s] not found on TestRun update", get_class($this), $id));
        }
        return $updated;
    }
}
